"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const plugin_commands_script_runners_1 = require("@pnpm/plugin-commands-script-runners");
const prepare_1 = require("@pnpm/prepare");
const cleanExpiredDlxCache_1 = require("./cleanExpiredDlxCache");
const createCacheKey = (...pkgs) => plugin_commands_script_runners_1.dlx.createCacheKey(pkgs, { default: 'https://registry.npmjs.com/' });
function createSampleDlxCacheLinkTarget(dirPath) {
    fs_1.default.mkdirSync(path_1.default.join(dirPath, 'node_modules', '.pnpm'), { recursive: true });
    fs_1.default.mkdirSync(path_1.default.join(dirPath, 'node_modules', '.bin'), { recursive: true });
    fs_1.default.writeFileSync(path_1.default.join(dirPath, 'node_modules', '.modules.yaml'), '');
    fs_1.default.writeFileSync(path_1.default.join(dirPath, 'package.json'), '');
    fs_1.default.writeFileSync(path_1.default.join(dirPath, 'pnpm-lock.yaml'), '');
}
function createSampleDlxCacheItem(cacheDir, cmd, now, age) {
    const hash = createCacheKey(cmd);
    const newDate = new Date(now.getTime() - age * 60_000);
    const timeError = 432; // just an arbitrary amount, nothing is special about this number
    const pid = 71014; // just an arbitrary number to represent pid
    const targetName = `${(newDate.getTime() - timeError).toString(16)}-${pid.toString(16)}`;
    const linkTarget = path_1.default.join(cacheDir, 'dlx', hash, targetName);
    const linkPath = path_1.default.join(cacheDir, 'dlx', hash, 'pkg');
    createSampleDlxCacheLinkTarget(linkTarget);
    fs_1.default.symlinkSync(linkTarget, linkPath, 'junction');
    fs_1.default.lutimesSync(linkPath, newDate, newDate);
}
function createSampleDlxCacheFsTree(cacheDir, now, ageTable) {
    for (const [cmd, age] of Object.entries(ageTable)) {
        createSampleDlxCacheItem(cacheDir, cmd, now, age);
    }
}
afterEach(() => {
    jest.restoreAllMocks();
});
test('cleanExpiredCache removes items that outlive dlxCacheMaxAge', async () => {
    (0, prepare_1.prepareEmpty)();
    const cacheDir = path_1.default.resolve('cache');
    const dlxCacheMaxAge = 7;
    const now = new Date();
    createSampleDlxCacheFsTree(cacheDir, now, {
        foo: 1,
        bar: 5,
        baz: 20,
    });
    const readdirSyncSpy = jest.spyOn(fs_1.default, 'readdirSync');
    const lstatSpy = jest.spyOn(fs_1.default.promises, 'lstat');
    const rmSpy = jest.spyOn(fs_1.default.promises, 'rm');
    await (0, cleanExpiredDlxCache_1.cleanExpiredDlxCache)({
        cacheDir,
        dlxCacheMaxAge,
        now,
    });
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'))).length).toBe(2);
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'))).length).toBe(2);
    expect(fs_1.default.existsSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('baz')))).toBeFalsy();
    expect(readdirSyncSpy).toHaveBeenCalledWith(path_1.default.join(cacheDir, 'dlx'), expect.anything());
    for (const key of ['foo', 'bar', 'baz']) {
        expect(lstatSpy).toHaveBeenCalledWith(path_1.default.join(cacheDir, 'dlx', createCacheKey(key), 'pkg'));
    }
    expect(rmSpy).not.toHaveBeenCalledWith(expect.stringContaining(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'))), expect.anything());
    expect(rmSpy).not.toHaveBeenCalledWith(expect.stringContaining(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'))), expect.anything());
    expect(rmSpy).toHaveBeenCalledWith(expect.stringContaining(path_1.default.join(cacheDir, 'dlx', createCacheKey('baz'))), { recursive: true, force: true });
    readdirSyncSpy.mockRestore();
    lstatSpy.mockRestore();
    rmSpy.mockRestore();
});
test('cleanExpiredCache removes all directories without checking stat if dlxCacheMaxAge is 0', async () => {
    (0, prepare_1.prepareEmpty)();
    const cacheDir = path_1.default.resolve('cache');
    const dlxCacheMaxAge = 0;
    const now = new Date();
    createSampleDlxCacheFsTree(cacheDir, now, {
        foo: 1,
        bar: 5,
        baz: 20,
    });
    const readdirSyncSpy = jest.spyOn(fs_1.default, 'readdirSync');
    const lstatSpy = jest.spyOn(fs_1.default.promises, 'lstat');
    const rmSpy = jest.spyOn(fs_1.default.promises, 'rm');
    await (0, cleanExpiredDlxCache_1.cleanExpiredDlxCache)({
        cacheDir,
        dlxCacheMaxAge,
        now,
    });
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx'))
        .sort()).toStrictEqual([]);
    expect(readdirSyncSpy).toHaveBeenCalledWith(path_1.default.join(cacheDir, 'dlx'), expect.anything());
    expect(lstatSpy).not.toHaveBeenCalled();
    for (const key of ['foo', 'bar', 'baz']) {
        expect(rmSpy).toHaveBeenCalledWith(path_1.default.join(cacheDir, 'dlx', createCacheKey(key)), { recursive: true, force: true });
    }
    readdirSyncSpy.mockRestore();
    lstatSpy.mockRestore();
    rmSpy.mockRestore();
});
test('cleanExpiredCache does nothing if dlxCacheMaxAge is Infinity', async () => {
    (0, prepare_1.prepareEmpty)();
    const cacheDir = path_1.default.resolve('cache');
    const dlxCacheMaxAge = Infinity;
    const now = new Date();
    createSampleDlxCacheFsTree(cacheDir, now, {
        foo: 1,
        bar: 5,
        baz: 20,
    });
    const readdirSpy = jest.spyOn(fs_1.default.promises, 'readdir');
    const lstatSpy = jest.spyOn(fs_1.default.promises, 'lstat');
    const rmSpy = jest.spyOn(fs_1.default.promises, 'rm');
    await (0, cleanExpiredDlxCache_1.cleanExpiredDlxCache)({
        cacheDir,
        dlxCacheMaxAge,
        now,
    });
    const dlxCacheDir = path_1.default.join(cacheDir, 'dlx');
    const entries = fs_1.default.readdirSync(dlxCacheDir).sort();
    expect(entries).toStrictEqual(['foo', 'bar', 'baz']
        .map(cmd => createCacheKey(cmd))
        .sort());
    for (const entry of entries) {
        expect(fs_1.default.readdirSync(path_1.default.join(dlxCacheDir, entry)).length).toBe(2);
    }
    expect(readdirSpy).not.toHaveBeenCalled();
    expect(lstatSpy).not.toHaveBeenCalled();
    expect(rmSpy).not.toHaveBeenCalled();
    readdirSpy.mockRestore();
    lstatSpy.mockRestore();
    rmSpy.mockRestore();
});
test("cleanOrphans deletes dirs that don't contain `link` and subdirs that aren't pointed to by `link` from the same parent", async () => {
    (0, prepare_1.prepareEmpty)();
    const cacheDir = path_1.default.resolve('cache');
    const now = new Date();
    // has link and orphans
    createSampleDlxCacheItem(cacheDir, 'foo', now, 0);
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'), `${now.getTime().toString(16)}-${(7000).toString(16)}`));
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'), `${now.getTime().toString(16)}-${(7005).toString(16)}`));
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'), `${now.getTime().toString(16)}-${(7102).toString(16)}`));
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'))).length).toBe(5);
    // has no link, only orphans
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'), `${now.getTime().toString(16)}-${(7000).toString(16)}`));
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'), `${now.getTime().toString(16)}-${(7005).toString(16)}`));
    createSampleDlxCacheLinkTarget(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'), `${now.getTime().toString(16)}-${(7102).toString(16)}`));
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar'))).length).toBe(3);
    await (0, cleanExpiredDlxCache_1.cleanOrphans)(path_1.default.join(cacheDir, 'dlx'));
    // expecting all subdirectories that aren't pointed to by `link` to be deleted.
    expect(fs_1.default.readdirSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('foo'))).length).toBe(2);
    // expecting directory that doesn't contain `link` to be deleted.
    expect(fs_1.default.existsSync(path_1.default.join(cacheDir, 'dlx', createCacheKey('bar')))).toBe(false);
});
//# sourceMappingURL=cleanExpiredDlxCache.test.js.map